@@ -19,25 +19,31 @@ Steps for signing a request:
    construct the base string
 5. Pass the base string and any keys needed to a signing function
 
-.. _`section 3.4`: http://tools.ietf.org/html/rfc5849#section-3.4
+.. _`section 3.4`: https://tools.ietf.org/html/rfc5849#section-3.4
 """
 from __future__ import absolute_import, unicode_literals
 
 import binascii
 import hashlib
 import hmac
+import logging
+
+from oauthlib.common import (extract_params, safe_string_equals,
+                             unicode_type, urldecode)
+
+from . import utils
+
 try:
     import urlparse
 except ImportError:
     import urllib.parse as urlparse
-from . import utils
-from oauthlib.common import urldecode, extract_params, safe_string_equals
-from oauthlib.common import bytes_type, unicode_type
+
+log = logging.getLogger(__name__)
 
 
-def construct_base_string(http_method, base_string_uri,
-        normalized_encoded_request_parameters):
-    """**String Construction**
+def signature_base_string(http_method, base_str_uri,
+                          normalized_encoded_request_parameters):
+    """**Construct the signature base string.**
     Per `section 3.4.1.1`_ of the spec.
 
     For example, the HTTP request::
@@ -64,7 +70,7 @@ def construct_base_string(http_method, base_string_uri,
         ethod%3DHMAC-SHA1%26oauth_timestamp%3D137131201%26oauth_token%3Dkkk
         9d7dh3k39sjv7
 
-    .. _`section 3.4.1.1`: http://tools.ietf.org/html/rfc5849#section-3.4.1.1
+    .. _`section 3.4.1.1`: https://tools.ietf.org/html/rfc5849#section-3.4.1.1
     """
 
     # The signature base string is constructed by concatenating together,
@@ -74,7 +80,7 @@ def construct_base_string(http_method, base_string_uri,
     #     "GET", "POST", etc.  If the request uses a custom HTTP method, it
     #     MUST be encoded (`Section 3.6`_).
     #
-    # .. _`Section 3.6`: http://tools.ietf.org/html/rfc5849#section-3.6
+    # .. _`Section 3.6`: https://tools.ietf.org/html/rfc5849#section-3.6
     base_string = utils.escape(http_method.upper())
 
     # 2.  An "&" character (ASCII code 38).
@@ -83,9 +89,9 @@ def construct_base_string(http_method, base_string_uri,
     # 3.  The base string URI from `Section 3.4.1.2`_, after being encoded
     #     (`Section 3.6`_).
     #
-    # .. _`Section 3.4.1.2`: http://tools.ietf.org/html/rfc5849#section-3.4.1.2
-    # .. _`Section 3.4.6`: http://tools.ietf.org/html/rfc5849#section-3.4.6
-    base_string += utils.escape(base_string_uri)
+    # .. _`Section 3.4.1.2`: https://tools.ietf.org/html/rfc5849#section-3.4.1.2
+    # .. _`Section 3.4.6`: https://tools.ietf.org/html/rfc5849#section-3.4.6
+    base_string += utils.escape(base_str_uri)
 
     # 4.  An "&" character (ASCII code 38).
     base_string += '&'
@@ -93,16 +99,16 @@ def construct_base_string(http_method, base_string_uri,
     # 5.  The request parameters as normalized in `Section 3.4.1.3.2`_, after
     #     being encoded (`Section 3.6`).
     #
-    # .. _`Section 3.4.1.3.2`: http://tools.ietf.org/html/rfc5849#section-3.4.1.3.2
-    # .. _`Section 3.4.6`: http://tools.ietf.org/html/rfc5849#section-3.4.6
+    # .. _`Section 3.4.1.3.2`: https://tools.ietf.org/html/rfc5849#section-3.4.1.3.2
+    # .. _`Section 3.4.6`: https://tools.ietf.org/html/rfc5849#section-3.4.6
     base_string += utils.escape(normalized_encoded_request_parameters)
 
     return base_string
 
 
-def normalize_base_string_uri(uri, host=None):
+def base_string_uri(uri, host=None):
     """**Base String URI**
-    Per `section 3.4.1.2`_ of the spec.
+    Per `section 3.4.1.2`_ of RFC 5849.
 
     For example, the HTTP request::
 
@@ -118,7 +124,7 @@ def normalize_base_string_uri(uri, host=None):
 
     is represented by the base string URI: "https://www.example.net:8080/".
 
-    .. _`section 3.4.1.2`: http://tools.ietf.org/html/rfc5849#section-3.4.1.2
+    .. _`section 3.4.1.2`: https://tools.ietf.org/html/rfc5849#section-3.4.1.2
 
     The host argument overrides the netloc part of the uri argument.
     """
@@ -132,7 +138,7 @@ def normalize_base_string_uri(uri, host=None):
     # are included by constructing an "http" or "https" URI representing
     # the request resource (without the query or fragment) as follows:
     #
-    # .. _`RFC3986`: http://tools.ietf.org/html/rfc3986
+    # .. _`RFC3986`: https://tools.ietf.org/html/rfc3986
 
     if not scheme or not netloc:
         raise ValueError('uri must include a scheme and netloc')
@@ -142,7 +148,7 @@ def normalize_base_string_uri(uri, host=None):
     # Note that the absolute path cannot be empty; if none is present in
     # the original URI, it MUST be given as "/" (the server root).
     #
-    # .. _`RFC 2616 section 5.1.2`: http://tools.ietf.org/html/rfc2616#section-5.1.2
+    # .. _`RFC 2616 section 5.1.2`: https://tools.ietf.org/html/rfc2616#section-5.1.2
     if not path:
         path = '/'
 
@@ -161,8 +167,8 @@ def normalize_base_string_uri(uri, host=None):
     #     to port 80 or when making an HTTPS request `RFC2818`_ to port 443.
     #     All other non-default port numbers MUST be included.
     #
-    # .. _`RFC2616`: http://tools.ietf.org/html/rfc2616
-    # .. _`RFC2818`: http://tools.ietf.org/html/rfc2818
+    # .. _`RFC2616`: https://tools.ietf.org/html/rfc2616
+    # .. _`RFC2818`: https://tools.ietf.org/html/rfc2818
     default_ports = (
         ('http', '80'),
         ('https', '443'),
@@ -172,7 +178,31 @@ def normalize_base_string_uri(uri, host=None):
         if (scheme, port) in default_ports:
             netloc = host
 
-    return urlparse.urlunparse((scheme, netloc, path, params, '', ''))
+    v = urlparse.urlunparse((scheme, netloc, path, params, '', ''))
+
+    # RFC 5849 does not specify which characters are encoded in the
+    # "base string URI", nor how they are encoded - which is very bad, since
+    # the signatures won't match if there are any differences. Fortunately,
+    # most URIs only use characters that are clearly not encoded (e.g. digits
+    # and A-Z, a-z), so have avoided any differences between implementations.
+    #
+    # The example from its section 3.4.1.2 illustrates that spaces in
+    # the path are percent encoded. But it provides no guidance as to what other
+    # characters (if any) must be encoded (nor how); nor if characters in the
+    # other components are to be encoded or not.
+    #
+    # This implementation **assumes** that **only** the space is percent-encoded
+    # and it is done to the entire value (not just to spaces in the path).
+    #
+    # This code may need to be changed if it is discovered that other characters
+    # are expected to be encoded.
+    #
+    # Note: the "base string URI" returned by this function will be encoded
+    # again before being concatenated into the "signature base string". So any
+    # spaces in the URI will actually appear in the "signature base string"
+    # as "%2520" (the "%20" further encoded according to section 3.6).
+
+    return v.replace(' ', '%20')
 
 
 # ** Request Parameters **
@@ -185,10 +215,10 @@ def normalize_base_string_uri(uri, host=None):
 #    particular manner that is often different from their original
 #    encoding scheme, and concatenated into a single string.
 #
-#    .. _`section 3.4.1.3`: http://tools.ietf.org/html/rfc5849#section-3.4.1.3
+# .. _`section 3.4.1.3`: https://tools.ietf.org/html/rfc5849#section-3.4.1.3
 
 def collect_parameters(uri_query='', body=[], headers=None,
-        exclude_oauth_signature=True, with_realm=False):
+                       exclude_oauth_signature=True, with_realm=False):
     """**Parameter Sources**
 
     Parameters starting with `oauth_` will be unescaped.
@@ -244,7 +274,7 @@ def collect_parameters(uri_query='', body=[], headers=None,
     parameter instances (the "a3" parameter is used twice in this
     request).
 
-    .. _`section 3.4.1.3.1`: http://tools.ietf.org/html/rfc5849#section-3.4.1.3.1
+    .. _`section 3.4.1.3.1`: https://tools.ietf.org/html/rfc5849#section-3.4.1.3.1
     """
     headers = headers or {}
     params = []
@@ -259,8 +289,8 @@ def collect_parameters(uri_query='', body=[], headers=None,
     #    and values and decoding them as defined by
     #    `W3C.REC-html40-19980424`_, Section 17.13.4.
     #
-    # .. _`RFC3986, Section 3.4`: http://tools.ietf.org/html/rfc3986#section-3.4
-    # .. _`W3C.REC-html40-19980424`: http://tools.ietf.org/html/rfc5849#ref-W3C.REC-html40-19980424
+    # .. _`RFC3986, Section 3.4`: https://tools.ietf.org/html/rfc3986#section-3.4
+    # .. _`W3C.REC-html40-19980424`: https://tools.ietf.org/html/rfc5849#ref-W3C.REC-html40-19980424
     if uri_query:
         params.extend(urldecode(uri_query))
 
@@ -269,7 +299,7 @@ def collect_parameters(uri_query='', body=[], headers=None,
     #    pairs excluding the "realm" parameter if present.  The parameter
     #    values are decoded as defined by `Section 3.5.1`_.
     #
-    # .. _`Section 3.5.1`: http://tools.ietf.org/html/rfc5849#section-3.5.1
+    # .. _`Section 3.5.1`: https://tools.ietf.org/html/rfc5849#section-3.5.1
     if headers:
         headers_lower = dict((k.lower(), v) for k, v in headers.items())
         authorization_header = headers_lower.get('authorization')
@@ -288,7 +318,7 @@ def collect_parameters(uri_query='', body=[], headers=None,
     #     *  The HTTP request entity-header includes the "Content-Type"
     #        header field set to "application/x-www-form-urlencoded".
     #
-    # .._`W3C.REC-html40-19980424`: http://tools.ietf.org/html/rfc5849#ref-W3C.REC-html40-19980424
+    # .._`W3C.REC-html40-19980424`: https://tools.ietf.org/html/rfc5849#ref-W3C.REC-html40-19980424
 
     # TODO: enforce header param inclusion conditions
     bodyparams = extract_params(body) or []
@@ -305,7 +335,7 @@ def collect_parameters(uri_query='', body=[], headers=None,
     # base string if present.
     if exclude_oauth_signature:
         unescaped_params = list(filter(lambda i: i[0] != 'oauth_signature',
-            unescaped_params))
+                                       unescaped_params))
 
     return unescaped_params
 
@@ -378,18 +408,18 @@ def normalize_parameters(params):
         dj82h48djs9d2&oauth_nonce=7d8f3e4a&oauth_signature_method=HMAC-SHA1
         &oauth_timestamp=137131201&oauth_token=kkk9d7dh3k39sjv7
 
-    .. _`section 3.4.1.3.2`: http://tools.ietf.org/html/rfc5849#section-3.4.1.3.2
+    .. _`section 3.4.1.3.2`: https://tools.ietf.org/html/rfc5849#section-3.4.1.3.2
     """
 
     # The parameters collected in `Section 3.4.1.3`_ are normalized into a
     # single string as follows:
     #
-    # .. _`Section 3.4.1.3`: http://tools.ietf.org/html/rfc5849#section-3.4.1.3
+    # .. _`Section 3.4.1.3`: https://tools.ietf.org/html/rfc5849#section-3.4.1.3
 
     # 1.  First, the name and value of each parameter are encoded
     #     (`Section 3.6`_).
     #
-    # .. _`Section 3.6`: http://tools.ietf.org/html/rfc5849#section-3.6
+    # .. _`Section 3.6`: https://tools.ietf.org/html/rfc5849#section-3.6
     key_values = [(utils.escape(k), utils.escape(v)) for k, v in params]
 
     # 2.  The parameters are sorted by name, using ascending byte value
@@ -409,10 +439,11 @@ def normalize_parameters(params):
 
 
 def sign_hmac_sha1_with_client(base_string, client):
-    return sign_hmac_sha1(base_string, 
-        client.client_secret,
-        client.resource_owner_secret
-    )
+    return sign_hmac_sha1(base_string,
+                          client.client_secret,
+                          client.resource_owner_secret
+                          )
+
 
 def sign_hmac_sha1(base_string, client_secret, resource_owner_secret):
     """**HMAC-SHA1**
@@ -424,8 +455,8 @@ def sign_hmac_sha1(base_string, client_secret, resource_owner_secret):
 
     Per `section 3.4.2`_ of the spec.
 
-    .. _`RFC2104`: http://tools.ietf.org/html/rfc2104
-    .. _`section 3.4.2`: http://tools.ietf.org/html/rfc5849#section-3.4.2
+    .. _`RFC2104`: https://tools.ietf.org/html/rfc2104
+    .. _`section 3.4.2`: https://tools.ietf.org/html/rfc5849#section-3.4.2
     """
 
     # The HMAC-SHA1 function variables are used in following way:
@@ -433,13 +464,13 @@ def sign_hmac_sha1(base_string, client_secret, resource_owner_secret):
     # text is set to the value of the signature base string from
     # `Section 3.4.1.1`_.
     #
-    # .. _`Section 3.4.1.1`: http://tools.ietf.org/html/rfc5849#section-3.4.1.1
+    # .. _`Section 3.4.1.1`: https://tools.ietf.org/html/rfc5849#section-3.4.1.1
     text = base_string
 
     # key is set to the concatenated values of:
     # 1.  The client shared-secret, after being encoded (`Section 3.6`_).
     #
-    # .. _`Section 3.6`: http://tools.ietf.org/html/rfc5849#section-3.6
+    # .. _`Section 3.6`: https://tools.ietf.org/html/rfc5849#section-3.6
     key = utils.escape(client_secret or '')
 
     # 2.  An "&" character (ASCII code 38), which MUST be included
@@ -448,7 +479,7 @@ def sign_hmac_sha1(base_string, client_secret, resource_owner_secret):
 
     # 3.  The token shared-secret, after being encoded (`Section 3.6`_).
     #
-    # .. _`Section 3.6`: http://tools.ietf.org/html/rfc5849#section-3.6
+    # .. _`Section 3.6`: https://tools.ietf.org/html/rfc5849#section-3.6
     key += utils.escape(resource_owner_secret or '')
 
     # FIXME: HMAC does not support unicode!
@@ -460,10 +491,76 @@ def sign_hmac_sha1(base_string, client_secret, resource_owner_secret):
     #         parameter, after the result octet string is base64-encoded
     #         per `RFC2045, Section 6.8`.
     #
-    # .. _`RFC2045, Section 6.8`: http://tools.ietf.org/html/rfc2045#section-6.8
+    # .. _`RFC2045, Section 6.8`: https://tools.ietf.org/html/rfc2045#section-6.8
     return binascii.b2a_base64(signature.digest())[:-1].decode('utf-8')
 
 
+def sign_hmac_sha256_with_client(base_string, client):
+    return sign_hmac_sha256(base_string,
+                          client.client_secret,
+                          client.resource_owner_secret
+                          )
+
+
+def sign_hmac_sha256(base_string, client_secret, resource_owner_secret):
+    """**HMAC-SHA256**
+
+    The "HMAC-SHA256" signature method uses the HMAC-SHA256 signature
+    algorithm as defined in `RFC4634`_::
+
+        digest = HMAC-SHA256 (key, text)
+
+    Per `section 3.4.2`_ of the spec.
+
+    .. _`RFC4634`: https://tools.ietf.org/html/rfc4634
+    .. _`section 3.4.2`: https://tools.ietf.org/html/rfc5849#section-3.4.2
+    """
+
+    # The HMAC-SHA256 function variables are used in following way:
+
+    # text is set to the value of the signature base string from
+    # `Section 3.4.1.1`_.
+    #
+    # .. _`Section 3.4.1.1`: https://tools.ietf.org/html/rfc5849#section-3.4.1.1
+    text = base_string
+
+    # key is set to the concatenated values of:
+    # 1.  The client shared-secret, after being encoded (`Section 3.6`_).
+    #
+    # .. _`Section 3.6`: https://tools.ietf.org/html/rfc5849#section-3.6
+    key = utils.escape(client_secret or '')
+
+    # 2.  An "&" character (ASCII code 38), which MUST be included
+    #     even when either secret is empty.
+    key += '&'
+
+    # 3.  The token shared-secret, after being encoded (`Section 3.6`_).
+    #
+    # .. _`Section 3.6`: https://tools.ietf.org/html/rfc5849#section-3.6
+    key += utils.escape(resource_owner_secret or '')
+
+    # FIXME: HMAC does not support unicode!
+    key_utf8 = key.encode('utf-8')
+    text_utf8 = text.encode('utf-8')
+    signature = hmac.new(key_utf8, text_utf8, hashlib.sha256)
+
+    # digest  is used to set the value of the "oauth_signature" protocol
+    #         parameter, after the result octet string is base64-encoded
+    #         per `RFC2045, Section 6.8`.
+    #
+    # .. _`RFC2045, Section 6.8`: https://tools.ietf.org/html/rfc2045#section-6.8
+    return binascii.b2a_base64(signature.digest())[:-1].decode('utf-8')
+
+_jwtrs1 = None
+
+#jwt has some nice pycrypto/cryptography abstractions
+def _jwt_rs1_signing_algorithm():
+    global _jwtrs1
+    if _jwtrs1 is None:
+        import jwt.algorithms as jwtalgo
+        _jwtrs1 = jwtalgo.RSAAlgorithm(jwtalgo.hashes.SHA1)
+    return _jwtrs1
+
 def sign_rsa_sha1(base_string, rsa_private_key):
     """**RSA-SHA1**
 
@@ -476,25 +573,22 @@ def sign_rsa_sha1(base_string, rsa_private_key):
     with the server that included its RSA public key (in a manner that is
     beyond the scope of this specification).
 
-    NOTE: this method requires the python-rsa library.
-
-    .. _`section 3.4.3`: http://tools.ietf.org/html/rfc5849#section-3.4.3
-    .. _`RFC3447, Section 8.2`: http://tools.ietf.org/html/rfc3447#section-8.2
+    .. _`section 3.4.3`: https://tools.ietf.org/html/rfc5849#section-3.4.3
+    .. _`RFC3447, Section 8.2`: https://tools.ietf.org/html/rfc3447#section-8.2
 
     """
-    # TODO: finish RSA documentation
-    from Crypto.PublicKey import RSA
-    from Crypto.Signature import PKCS1_v1_5
-    from Crypto.Hash import SHA
-    key = RSA.importKey(rsa_private_key)
     if isinstance(base_string, unicode_type):
         base_string = base_string.encode('utf-8')
-    h = SHA.new(base_string)
-    p = PKCS1_v1_5.new(key)
-    return binascii.b2a_base64(p.sign(h))[:-1].decode('utf-8')
+    # TODO: finish RSA documentation
+    alg = _jwt_rs1_signing_algorithm()
+    key = _prepare_key_plus(alg, rsa_private_key)
+    s=alg.sign(base_string, key)
+    return binascii.b2a_base64(s)[:-1].decode('utf-8')
 
 
 def sign_rsa_sha1_with_client(base_string, client):
+    if not client.rsa_key:
+        raise ValueError('rsa_key is required when using RSA signature method.')
     return sign_rsa_sha1(base_string, client.rsa_key)
 
 
@@ -509,7 +603,7 @@ def sign_plaintext(client_secret, resource_owner_secret):
     utilize the signature base string or the "oauth_timestamp" and
     "oauth_nonce" parameters.
 
-    .. _`section 3.4.4`: http://tools.ietf.org/html/rfc5849#section-3.4.4
+    .. _`section 3.4.4`: https://tools.ietf.org/html/rfc5849#section-3.4.4
 
     """
 
@@ -518,7 +612,7 @@ def sign_plaintext(client_secret, resource_owner_secret):
 
     # 1.  The client shared-secret, after being encoded (`Section 3.6`_).
     #
-    # .. _`Section 3.6`: http://tools.ietf.org/html/rfc5849#section-3.6
+    # .. _`Section 3.6`: https://tools.ietf.org/html/rfc5849#section-3.6
     signature = utils.escape(client_secret or '')
 
     # 2.  An "&" character (ASCII code 38), which MUST be included even
@@ -527,7 +621,7 @@ def sign_plaintext(client_secret, resource_owner_secret):
 
     # 3.  The token shared-secret, after being encoded (`Section 3.6`_).
     #
-    # .. _`Section 3.6`: http://tools.ietf.org/html/rfc5849#section-3.6
+    # .. _`Section 3.6`: https://tools.ietf.org/html/rfc5849#section-3.6
     signature += utils.escape(resource_owner_secret or '')
 
     return signature
@@ -536,13 +630,14 @@ def sign_plaintext(client_secret, resource_owner_secret):
 def sign_plaintext_with_client(base_string, client):
     return sign_plaintext(client.client_secret, client.resource_owner_secret)
 
+
 def verify_hmac_sha1(request, client_secret=None,
-    resource_owner_secret=None):
+                     resource_owner_secret=None):
     """Verify a HMAC-SHA1 signature.
 
     Per `section 3.4`_ of the spec.
 
-    .. _`section 3.4`: http://tools.ietf.org/html/rfc5849#section-3.4
+    .. _`section 3.4`: https://tools.ietf.org/html/rfc5849#section-3.4
 
     To satisfy `RFC2616 section 5.2`_ item 1, the request argument's uri
     attribute MUST be an absolute URI whose netloc part identifies the
@@ -550,25 +645,65 @@ def verify_hmac_sha1(request, client_secret=None,
     item of the request argument's headers dict attribute will be
     ignored.
 
-    .. _`RFC2616 section 5.2`: http://tools.ietf.org/html/rfc2616#section-5.2
+    .. _`RFC2616 section 5.2`: https://tools.ietf.org/html/rfc2616#section-5.2
 
     """
     norm_params = normalize_parameters(request.params)
-    uri = normalize_base_string_uri(request.uri)
-    base_string = construct_base_string(request.http_method, uri, norm_params)
-    signature = sign_hmac_sha1(base_string, client_secret,
-        resource_owner_secret)
-    return safe_string_equals(signature, request.signature)
+    bs_uri = base_string_uri(request.uri)
+    sig_base_str = signature_base_string(request.http_method, bs_uri,
+                                         norm_params)
+    signature = sign_hmac_sha1(sig_base_str, client_secret,
+                               resource_owner_secret)
+    match = safe_string_equals(signature, request.signature)
+    if not match:
+        log.debug('Verify HMAC-SHA1 failed: signature base string: %s',
+                  sig_base_str)
+    return match
 
 
+def verify_hmac_sha256(request, client_secret=None,
+                     resource_owner_secret=None):
+    """Verify a HMAC-SHA256 signature.
+
+    Per `section 3.4`_ of the spec.
+
+    .. _`section 3.4`: https://tools.ietf.org/html/rfc5849#section-3.4
+
+    To satisfy `RFC2616 section 5.2`_ item 1, the request argument's uri
+    attribute MUST be an absolute URI whose netloc part identifies the
+    origin server or gateway on which the resource resides. Any Host
+    item of the request argument's headers dict attribute will be
+    ignored.
+
+    .. _`RFC2616 section 5.2`: https://tools.ietf.org/html/rfc2616#section-5.2
+
+    """
+    norm_params = normalize_parameters(request.params)
+    bs_uri = base_string_uri(request.uri)
+    sig_base_str = signature_base_string(request.http_method, bs_uri,
+                                         norm_params)
+    signature = sign_hmac_sha256(sig_base_str, client_secret,
+                               resource_owner_secret)
+    match = safe_string_equals(signature, request.signature)
+    if not match:
+        log.debug('Verify HMAC-SHA256 failed: signature base string: %s',
+                  sig_base_str)
+    return match
+
+
+def _prepare_key_plus(alg, keystr):
+    if isinstance(keystr, bytes):
+        keystr = keystr.decode('utf-8')
+    return alg.prepare_key(keystr)
+
 def verify_rsa_sha1(request, rsa_public_key):
     """Verify a RSASSA-PKCS #1 v1.5 base64 encoded signature.
 
     Per `section 3.4.3`_ of the spec.
 
-    Note this method requires the PyCrypto library.
+    Note this method requires the jwt and cryptography libraries.
 
-    .. _`section 3.4.3`: http://tools.ietf.org/html/rfc5849#section-3.4.3
+    .. _`section 3.4.3`: https://tools.ietf.org/html/rfc5849#section-3.4.3
 
     To satisfy `RFC2616 section 5.2`_ item 1, the request argument's uri
     attribute MUST be an absolute URI whose netloc part identifies the
@@ -576,19 +711,22 @@ def verify_rsa_sha1(request, rsa_public_key):
     item of the request argument's headers dict attribute will be
     ignored.
 
-    .. _`RFC2616 section 5.2`: http://tools.ietf.org/html/rfc2616#section-5.2
+    .. _`RFC2616 section 5.2`: https://tools.ietf.org/html/rfc2616#section-5.2
     """
-    from Crypto.PublicKey import RSA
-    from Crypto.Signature import PKCS1_v1_5
-    from Crypto.Hash import SHA
-    key = RSA.importKey(rsa_public_key)
     norm_params = normalize_parameters(request.params)
-    uri = normalize_base_string_uri(request.uri)
-    message = construct_base_string(request.http_method, uri, norm_params)
-    h = SHA.new(message.encode('utf-8'))
-    p = PKCS1_v1_5.new(key)
+    bs_uri = base_string_uri(request.uri)
+    sig_base_str = signature_base_string(request.http_method, bs_uri,
+                                         norm_params).encode('utf-8')
     sig = binascii.a2b_base64(request.signature.encode('utf-8'))
-    return p.verify(h, sig)
+
+    alg = _jwt_rs1_signing_algorithm()
+    key = _prepare_key_plus(alg, rsa_public_key)
+
+    verify_ok = alg.verify(sig_base_str, key, sig)
+    if not verify_ok:
+        log.debug('Verify RSA-SHA1 failed: signature base string: %s',
+                  sig_base_str)
+    return verify_ok
 
 
 def verify_plaintext(request, client_secret=None, resource_owner_secret=None):
@@ -596,7 +734,10 @@ def verify_plaintext(request, client_secret=None, resource_owner_secret=None):
 
     Per `section 3.4`_ of the spec.
 
-    .. _`section 3.4`: http://tools.ietf.org/html/rfc5849#section-3.4
+    .. _`section 3.4`: https://tools.ietf.org/html/rfc5849#section-3.4
     """
     signature = sign_plaintext(client_secret, resource_owner_secret)
-    return safe_string_equals(signature, request.signature)
+    match = safe_string_equals(signature, request.signature)
+    if not match:
+        log.debug('Verify PLAINTEXT failed')
+    return match
